@@ -1,21 +1,11 @@
-use crate::{AsciiCase, Error, ERROR};
+#![allow(missing_docs)]
 
-use simd_abstraction::generic::hex::unhex;
+use crate::sa_hex::unhex;
+use crate::{AsciiCase, Error, Hex, ERROR};
 
 #[inline(always)]
 const fn shl4(x: u8) -> u8 {
-    const SHL4_TABLE: [u8; 256] = {
-        let mut buf = [0; 256];
-        let mut i: u8 = 0;
-        loop {
-            buf[i as usize] = i.wrapping_shl(4);
-            if i == 255 {
-                break buf;
-            }
-            i += 1;
-        }
-    };
-    SHL4_TABLE[x as usize]
+    x.wrapping_shl(4)
 }
 
 const UPPER_TABLE: &[u8; 16] = b"0123456789ABCDEF";
@@ -93,7 +83,7 @@ const fn char_lut(case: AsciiCase) -> &'static [u8; 16] {
 }
 
 #[inline]
-pub const fn format_simple(src: &[u8; 16], case: AsciiCase) -> [u8; 32] {
+pub const fn format_simple(src: &[u8; 16], case: AsciiCase) -> Hex<32> {
     let lut = char_lut(case);
     let mut dst = [0; 32];
     let mut i = 0;
@@ -103,11 +93,11 @@ pub const fn format_simple(src: &[u8; 16], case: AsciiCase) -> [u8; 32] {
         dst[i * 2 + 1] = lut[(x & 0x0f) as usize];
         i += 1;
     }
-    dst
+    unsafe { Hex::new_unchecked(dst) }
 }
 
 #[inline]
-pub const fn format_hyphenated(src: &[u8; 16], case: AsciiCase) -> [u8; 36] {
+pub const fn format_hyphenated(src: &[u8; 16], case: AsciiCase) -> Hex<36> {
     let lut = char_lut(case);
     let groups = [(0, 8), (9, 13), (14, 18), (19, 23), (24, 36)];
     let mut dst = [0; 36];
@@ -130,7 +120,7 @@ pub const fn format_hyphenated(src: &[u8; 16], case: AsciiCase) -> [u8; 36] {
         }
         group_idx += 1;
     }
-    dst
+    unsafe { Hex::new_unchecked(dst) }
 }
 
 #[test]
