@@ -1,4 +1,4 @@
-use crate::{AsciiCase, Error};
+use crate::{AsciiCase, Error, Hex};
 
 fn ok_cases() -> &'static [(&'static str, &'static str)] {
     const A1: &str = "67e5504410b1426f9247bb680e5fe0c8";
@@ -88,28 +88,28 @@ fn format_cases() -> &'static [(&'static str, &'static str)] {
     ]
 }
 
-pub fn test_format_simple(f: impl Fn(&[u8; 16], AsciiCase) -> [u8; 32]) {
+pub fn test_format_simple(f: impl Fn(&[u8; 16], AsciiCase) -> Hex<32>) {
     for &(input, _) in format_cases() {
         let mut src: [u8; 16] = [0; 16];
         faster_hex::hex_decode(input.as_bytes(), &mut src).unwrap();
 
         let buf = f(&src, AsciiCase::Upper);
-        assert_eq!(&buf, input.to_ascii_uppercase().as_bytes());
+        assert_eq!(buf.as_bytes(), input.to_ascii_uppercase().as_bytes());
 
         let buf = f(&src, AsciiCase::Lower);
-        assert_eq!(&buf, input.to_ascii_lowercase().as_bytes());
+        assert_eq!(buf.as_bytes(), input.to_ascii_lowercase().as_bytes());
     }
 }
 
-pub fn test_format_hypenated(f: impl Fn(&[u8; 16], AsciiCase) -> [u8; 36]) {
+pub fn test_format_hypenated(f: impl Fn(&[u8; 16], AsciiCase) -> Hex<36>) {
     for &(input, expected) in format_cases() {
         let mut src: [u8; 16] = [0; 16];
         faster_hex::hex_decode(input.as_bytes(), &mut src).unwrap();
 
         let buf = f(&src, AsciiCase::Upper);
-        assert_eq!(&buf, expected.to_ascii_uppercase().as_bytes());
+        assert_eq!(buf.as_bytes(), expected.to_ascii_uppercase().as_bytes());
 
         let buf = f(&src, AsciiCase::Lower);
-        assert_eq!(&buf, expected.to_ascii_lowercase().as_bytes());
+        assert_eq!(buf.as_bytes(), expected.to_ascii_lowercase().as_bytes());
     }
 }
